//|------------------------------------------------------------------+
//|																 											 SOM_ex1-hex |
//|							  				 Copyright 2011, MetaQuotes Software Corp. |
//|																							 http://www.mql5.com |
//|------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link		"http://www.mql5.com"
#property version	"1.00"

//------------------------------------------------------------------	CSOMNode
class CSOMNode //   SOM-
{
protected:
	int	m_x1;
	int	m_y1;
	int	m_x2;
	int	m_y2;
	double m_x;
	double m_y;
	double m_weights[];
	int m_dimension;
public:
	CSOMNode(); //  
	~CSOMNode(); //  
	void InitNode(int x1, int y1, int x2, int y2); //    
	double X() const { return(m_x); } //     
	double Y() const { return(m_y); }
	void GetCoordinates(int &x1,int &y1,int &x2,int &y2); //    
	double GetWeight(int weight_index); //     weight_index
	double CalculateDistance(double &vector[]); //       
	void AdjustWeights(double &vector[],double learning_rate,double influence); //    
	void SetDimension(int dimension); //    
};
//------------------------------------------------------------------	CSOMNode
void CSOMNode::CSOMNode() //  
{
	m_x1=0; m_y1=0; m_x2=0; m_y2=0; m_x=0.0; m_y=0.0;
}
//------------------------------------------------------------------	InitNode
void CSOMNode::InitNode(int x1, int y1, int x2, int y2) //   
{
	m_x1=x1; m_y1=y1; m_x2=x2; m_y2=y2; //   
	m_x=x1+MathAbs((x2-x1))/2; m_y=y1+MathAbs((y2-y1))/2; //    
	ArrayResize(m_weights,m_dimension); //   
	for(int i=0; i<m_dimension; i++) m_weights[i]=255.0*rand()/32768; //    ;
}
//------------------------------------------------------------------	~CSOMNode
void CSOMNode::~CSOMNode() //  
{
	ArrayResize(m_weights,0); 
}
//------------------------------------------------------------------	GetWeight
double CSOMNode::GetWeight(int weight_index) //     weight_index
{
	if(weight_index>=0 && weight_index<m_dimension) return(m_weights[weight_index]); else return(0);
}
//------------------------------------------------------------------	GetCoordinates
void CSOMNode::GetCoordinates(int &x1,int &y1,int &x2,int &y2) //    
{
	x1=m_x1; y1=m_y1; x2=m_x2; y2=m_y2;
}
//------------------------------------------------------------------	CalculateDistance
double CSOMNode::CalculateDistance(double &vector[]) //        
{
	double dist_sqr=0;
	for(int i=0; i<m_dimension; i++) dist_sqr=dist_sqr+(vector[i]-m_weights[i])*(vector[i]-m_weights[i]);
	return(dist_sqr);
}
//------------------------------------------------------------------	AdjustWeights
void CSOMNode::AdjustWeights(double &vector[],double learning_rate,double influence) //       
{
	for(int i=0; i<m_dimension; i++) m_weights[i]=m_weights[i]+learning_rate*influence*(vector[i]-m_weights[i]);
}
//------------------------------------------------------------------	SetDimension
void CSOMNode::SetDimension(int dimension)
{
	m_dimension=dimension;
}